from abc import ABC, abstractmethod
import os

class Template(ABC):
    def __init__(self):
        self.content = None
        self.file_path = None

    @abstractmethod
    def set_template(self, content):
        """
        Set the content of the template.

        Parameters:
            content (str): The content of the template.
        """
        pass

    def set_file_path(self, file_path):
        """
        Set the file path of the template.

        Parameters:
            file_path (str): The file path of the template.
        """
        self.file_path = file_path        
        try:  
            with open(self.file_path, 'r') as file:
                self.content = file.read()             
        except Exception as e:
             print({'status': 'Template adding Failure', 'error_message': f"error: {str(e)}"})

        return self

    @abstractmethod
    def set_value(self, key, value):
        """
        Set additional values for the template.

        Parameters:
            key (str): The key of the value.
            value (any): The value to be set.
        """
        pass

    @abstractmethod
    def build(self):
        """
        Build the template.
        This method should be overridden in subclasses if additional processing is needed.
        """
        pass

class HTMLTemplate(Template):
    def __init__(self):
        super().__init__()
        self.content_type = "text/html"

    def set_template(self, content):
        """
        Set the content of the HTML template.

        Parameters:
            content (str): The content of the HTML template.
        """
        # You can perform additional processing specific to HTML templates here if needed
        self.content = content
        return self

    def set_value(self, key, value):
        """
        Set additional values for the HTML template.

        Parameters:
            key (str): The key of the value.
            value (any): The value to be set.
        """
        if self.content:
            self.content = self.content.replace("{{" + key + "}}", str(value))
        return self

    def build(self):
        """
        Build the HTML template.
        This method reads the content from the file path and sets it as the template content.
        """
        pass

# # Example usage:
# html_template = HTMLTemplate()
# html_template.set_file_path('payomail/template/test.html')
# html_template.set_value('name', 'John Doe')
# html_template.set_value('age', '123 Main Street')
# print(html_template.content)  # Output the modified HTML content with replaced values



# main code

from datetime import datetime
import os
from typing import  Union
from urllib.parse import urlparse
from payomail.core.strategy import EmailStrategy
from payomail.core.file import download_file, get_size_by_path
from payomail.template.template import HTMLTemplate


class EmailBuilder:
    def __init__(self, default_max_attachment_size_mb=10):
        self.email = Email(default_max_attachment_size_mb)
        self.pending_attachments = []  # to store attachments that are added after building

    def set_strategy(self, strategy: EmailStrategy):
        self.email.strategy = strategy
        return self

    def set_sender(self, sender_email):
        self.email.sender_email = sender_email
        return self

    def set_password(self, app_password):
        self.email.app_password = app_password
        return self

    def set_recipient(self, recipient_email):
        self.email.recipient_email = recipient_email
        return self

    def set_subject(self, subject):
        self.email.subject = subject
        return self

    def set_body(self, body):
        self.email.body = body
        return self

    def set_max_attachment_size(self, max_attachment_size_mb):
        self.email.set_max_attachment_size(max_attachment_size_mb)
        return self
           
    def build(self):
        return self.email


class Email:
    def __init__(self, default_max_attachment_size_mb=10):
        self.strategy = None
        self.sender_email = None
        self.app_password = None
        self.recipient_email = None
        self.subject = None
        self.body = None
        self.html_content=None
        self.attachments = []
        self.max_attachment_size_bytes = default_max_attachment_size_mb * 1024 * 1024  # Default: 10 MB

    def set_max_attachment_size(self, max_attachment_size_mb):
        self.max_attachment_size_bytes = max_attachment_size_mb * 1024 * 1024  # Convert MB to bytes

    def attach_file(self, file_source: Union[str, bytes]):
        if isinstance(file_source, str) and urlparse(file_source).scheme:
            result = download_file(file_source, self.max_attachment_size_bytes)
            if result['status'] == 'Failure':
                print(f"Failed to attach file. Error message: {result['error_message']}")
            self.attachments.append(result['path'])
        else:
            result = get_size_by_path(file_source)
            if result['size'] > self.max_attachment_size_bytes:
               print(f"Failed to attach file. Error message: {result['error_message']}")
               file_source=''
            self.attachments.append(file_source)
        return self


    def send(self):
        if self.strategy:
            try:
                self.strategy.send_email(
                    self.sender_email, self.app_password, self.recipient_email,
                    self.subject, self.body,self.html_content, self.attachments
                )
                self.clean_temp()
                timestamp = datetime.now().strftime("%d/%m/%Y %H:%M")
                response = {
                    'status': 'Success',
                    'from': self.sender_email,
                    'to': self.recipient_email,
                    'subject': self.subject,
                    'timestamp': timestamp
                }
                return response

            except Exception as e:
                self.clean_temp()
                timestamp = datetime.now().strftime("%d/%m/%Y %H:%M")
                response = {
                    'status': 'Failure',
                    'error_message': str(e),
                    'from': self.sender_email,
                    'to': self.recipient_email,
                    'subject': self.subject,
                    'timestamp': timestamp
                }
                return response                                    
        else:
            timestamp = datetime.now().strftime("%d/%m/%Y %H:%M")
            response = {
                'status': 'Failure',
                'error_message': 'Strategy not set. Use set_strategy() to set the email strategy.',
                'from': None,
                'to': None,
                'subject': None,
                'timestamp': timestamp
            }
            return response

    def set_subject(self, subject):
        self.subject = subject
        return self

    def set_body(self, body):
        self.body = body
        return self

    def set_recipient(self, recipient_email):
        self.recipient_email = recipient_email
        return self
    
    def clean_temp(self):
        temp_folder = os.path.join(os.getcwd(), 'payomail', 'temp')
        for attachment in self.attachments:
            try:
                attachment_path = os.path.abspath(attachment)
                if os.path.commonpath([attachment_path, temp_folder]) == temp_folder:
                   os.remove(attachment_path)
            except (FileNotFoundError, OSError):
                   print(f"Error removing file: {attachment}")
   
    def set_body_from_template(self, template):
        """
        Set the email body from a template.

        Parameters:
            template (Template): The template object containing the content.
        """
        if isinstance(template, HTMLTemplate):
            # If the template is an HTML template, set the body accordingly
            self.html_content = template.content
        return self


from abc import ABC, abstractmethod
from email import encoders
from email.mime.base import MIMEBase
import os
import smtplib
from email.mime.text import MIMEText
from email.mime.multipart import MIMEMultipart
from email.mime.application import MIMEApplication
from os.path import basename

class EmailStrategy(ABC):
    @abstractmethod
    def send_email(self, sender_email, app_password, recipient_email, subject, body,html_content, attachments=None):
        pass

class GmailStrategy(EmailStrategy):
    def send_email(self, sender_email, app_password, recipient_email, subject, body,html_content, attachments=None):
        smtp_server = "smtp.gmail.com"
        smtp_port = 587
        server = smtplib.SMTP(smtp_server, smtp_port)
        server.starttls()
        server.login(sender_email, app_password)
        self._send_email(server, sender_email, recipient_email, subject, body,html_content, attachments)

    def _send_email(self, server, sender_email, recipient_email, subject, body,html_content, attachments):
        message = MIMEMultipart()
        message["From"] = sender_email
        message["To"] = recipient_email
        message["Subject"] = subject
        if body is not None:
            message.attach(MIMEText(body, "plain"))
        if html_content is not None:
            html_part = MIMEText(html_content, 'html')
            message.attach(html_part)

        for attachment in attachments:
            with open(attachment, "rb") as file:
                part = MIMEApplication(file.read(), Name=os.path.split(attachment)[-1])
                part['Content-Disposition'] = f'attachment; filename="{os.path.split(attachment)[-1]}"'
                message.attach(part)

        server.sendmail(sender_email, recipient_email, message.as_string())
        
class IceWarpStrategy(EmailStrategy):
    def send_email(self, sender_email, app_password, recipient_email, subject, body,html_content, attachments):
        smtp_server = "mail.microproindia.com"
        smtp_port = 587
        server = smtplib.SMTP(smtp_server, smtp_port)
        server.starttls()
        server.login(sender_email, app_password)
        self._send_email(server, sender_email, recipient_email, subject, body,html_content, attachments)

    def _send_email(self, server, sender_email, recipient_email, subject, body,html_content, attachments):
        message = MIMEMultipart()
        message["From"] = sender_email
        message["To"] = recipient_email
        message["Subject"] = subject
        if body is not None:
            message.attach(MIMEText(body, "plain"))
        if html_content is not None:
            html_part = MIMEText(html_content, 'html')
            message.attach(html_part)

        for attachment in attachments:
            with open(attachment, "rb") as file:
                part = MIMEApplication(file.read(), Name=os.path.split(attachment)[-1])
                part['Content-Disposition'] = f'attachment; filename="{os.path.split(attachment)[-1]}"'
                message.attach(part)

        server.sendmail(sender_email, recipient_email, message.as_string())


from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name='payomail',
    version='9.0.0',
    packages=find_packages(),
    package_data={'': ['icon.png']},  # Include all PNG files in the package
    include_package_data=True,
    # Metadata
    project_urls={
        'Icon': 'https://github.com/Roshangedam/payomail/blob/master/payomail/images/icon.png',
    },
    author='Roshan Gedam',
    author_email='roshangedam1998@gmail.com',
    description='mailing utilty',
    url='https://github.com/Roshangedam/payomail',
    long_description=long_description,
    long_description_content_type="text/markdown",
)

# for add in publish pacakge use commands

# py setup.py sdist

# twine upload dist/*

# give your api token key

# done


import os
import requests
from urllib.parse import urlparse, unquote
from sre_constants import MAGIC
from datetime import datetime

def get_file_type_from_url(url):
    try:
        response = requests.head(url)
        content_type = response.headers.get('content-type')

        if content_type:
            if 'application/pdf' in content_type:
                return 'pdf'
            elif 'image' in content_type:
                return 'png'
            elif 'video' in content_type:
                return 'mp4'
            elif 'audio' in content_type:
                return 'mp3'
            else:
                return ''
        else:
            with requests.get(url, stream=True) as r:
                return detect_file_type(r.content)
    except requests.exceptions.RequestException as e:
        print(f"Error fetching URL: {e}")
        return 'Unknown'

def detect_file_type(file_content):
    file_type = MAGIC.Magic(mime=True).from_buffer(file_content)
    if 'application/pdf' in file_type:
        return 'pdf'
    elif 'image' in file_type:
        return 'png'
    elif 'video' in file_type:
        return 'mp4'
    elif 'audio' in file_type:
        return 'mp3'
    else:
        return ''

def download_file(file_source, max_attachment_size_bytes):
    try:
        # Determine file type and generate filename based on URL
        file_type = get_file_type_from_url(file_source)
        parsed_url = urlparse(file_source)
        timestamp = datetime.now().strftime('%Y%m%d%H%M%S')
        filename = None
        
        # If the URL contains a filename with extension, use it
        if parsed_url.path and '.' in parsed_url.path:
            filename = os.path.basename(unquote(parsed_url.path))
            filename = f"{os.path.splitext(filename)[0]}_{timestamp}{os.path.splitext(filename)[1]}"
        
        # If the response has a filename, use it
        if 'filename' in parsed_url.params:
            filename = parsed_url.params['filename']
        
        # If neither the URL nor the response contains a filename, generate one
        if not filename:
            filename = f"unknown_{timestamp}.{file_type.lower()}"
        
        # Download the file
        response = requests.get(file_source)
        response.raise_for_status()
        file_data = response.content
        
        # Save the file with the generated filename
        temp_folder = os.path.join(os.getcwd(), 'payomail', 'temp')
        os.makedirs(temp_folder, exist_ok=True)
        file_path = os.path.join(temp_folder, filename)

        if len(file_data) > max_attachment_size_bytes:
            return {'status': 'Failure', 'path': '', 'error_message': f"File size exceeds the maximum allowed size ({max_attachment_size_bytes} bytes)."}

        with open(file_path, 'wb') as file:
            file.write(file_data)

        return {'status': 'Success', 'path': file_path, 'error_message': ''}
    
    except requests.exceptions.RequestException as e:
        return {'status': 'Failure', 'path': '', 'error_message': f"Error downloading file: {str(e)}"}
    
    except Exception as e:
        return {'status': 'Failure', 'path': '', 'error_message': f"An unexpected error occurred: {str(e)}"}

def get_size_by_path(file_path):
    try:
        os.path.exists(file_path)
        size:int
        return {'status': 'Failure', 'size': os.path.getsize(file_path), 'error_message': f"error: {str(e)}"}
    except Exception as e:
        return {'status': 'Failure', 'size': -1, 'error_message': f"error: {str(e)}"}
